/*
   Copyright (C) 2013  PIERRE Fabien.
   All rights reserved.
   Read license.txt.
*/

#include "matrix.h"
#include "mex.h"

#include <stdio.h>
#include <stdlib.h>

#include <math.h>

/*************************************************************************
 *
 * AUTEUR : Fabien PIERRE.
 * DATE   : 9 juillet 2013.
 *
 *
 * Compilation :
 
  % linux
  clc
  mex CFLAGS="\$CFLAGS -std=c99" block_DCT.c -output block_DCT
 
  % windows
  clc
  mex MYdct.c -output MYdct
 
 
 ************************************************************************/

void mexFunction(int nlhs, mxArray *plhs[]       ,
        int nrhs, const mxArray *prhs[]  )
{
    /* Declarations.*/
    double *x,*X; /* , *WT;*/
    int ndim, i,j;
    const int *dims;
    double C1, C2, C3, C4, C5, C6, C7, SP1, SP2, SP3, SP4; 
    double in[8];
    
    /* Verification du nombre d'arguments.*/
    if(nrhs != 1)
    {
        mexErrMsgIdAndTxt("block_DCT:nrhs","1 inputs required.");
    }
    if(nlhs != 1)
    {
        mexErrMsgIdAndTxt("block_DCT:nlhs","1 output required.");
    }
    
    /* Gestion entrees-sorties.*/
    ndim= mxGetNumberOfDimensions(prhs[0]);
    dims= mxGetDimensions(prhs[0]);
    
    C1= 0.490392640201615 ;
    C2= 0.461939766255643 ;
    C3= 0.415734806151273 ;
    C4= 0.353553390593274 ;
    C5= 0.277785116509801 ;
    C6= 0.191341716182545 ;
    C7= 0.097545161008064 ;
    
    x = (const double*)(mxGetPr(prhs[0]));
    
    int height = dims[0];
    int width  = dims[1];
    
    int Height = 8 * (height / 8);
    int Width  = 8 * (width  / 8);
        
    int dims2[2];
    dims2[0] = Height  ;
    dims2[1] = Width  ;
    
    int height_Block = Height / 8;
    int width_Block = Width / 8;
    int length_Block = height_Block *width_Block ;
    
        
    plhs[0]= mxCreateNumericArray(ndim, dims2 ,
            mxDOUBLE_CLASS, mxREAL     ) ;
    X= (double*)mxGetPr(plhs[0]);
    for(int lin=0; lin < Height; lin++){
        for(int col=0; col < Width; col++){
            X[ lin + col * Height]= x[lin + col * height];
        }
    }
    
    int X_Block;
    int Y_Block;
    int offset;
    
    for(int index= 0; index<length_Block; index ++){
        
        X_Block= index % height_Block;
        Y_Block= (index - X_Block) / height_Block;
        offset= X_Block * 8 + 64 * height_Block * Y_Block;
        
        for(int c=0; c< 8; c++){
            
            for(i=0; i< 4;  i++){
                j= 8 - i - 1;
                in[i    ] = X[i + c*Height + offset] 
                        + X[j + c*Height + offset];
                in[i + 4] = X[i + c*Height + offset] 
                        - X[j + c*Height + offset];
            }
            
            SP1= C4 * (in[0] + in[3]);
            SP2= C4 * (in[1] + in[2]);
            X[ c*Height + offset]= SP1 + SP2;
            X[4 + c*Height + offset]= SP1 - SP2;
            SP3= in[0] - in[3];
            SP4= in[1] - in[2];
            X[2 + c*Height + offset]=   C2 * SP3 + C6 * SP4;
            X[6 + c*Height + offset]= - C2 * SP4 + C6 * SP3;
            X[1 + c*Height + offset]= C1 * in[4] + C3 * in[5] 
                    + C5 * in[6] + C7 * in[7];
            X[3 + c*Height + offset]= C3 * in[4] - C7 * in[5] 
                    - C1 * in[6] - C5 * in[7];
            X[5 + c*Height + offset]= C5 * in[4] - C1 * in[5] 
                    + C7 * in[6] + C3 * in[7];
            X[7 + c*Height + offset]= C7 * in[4] - C5 * in[5] 
                    + C3 * in[6] - C1 * in[7];
            
        }
        
        for(int c=0; c< 8; c++){
            for(i=0; i< 4;  i++){
                j= 8 - i - 1;
                in[i    ] = X[c + i*Height + offset] 
                        + X[c + j*Height + offset];
                in[i + 4] = X[c + i*Height + offset] 
                        - X[c + j*Height + offset];
            }
            
            SP1= C4 * (in[0] + in[3]);
            SP2= C4 * (in[1] + in[2]);
            X[c + offset]= SP1 + SP2;
            X[c + 4*Height + offset]= SP1 - SP2;
            SP3= in[0] - in[3];
            SP4= in[1] - in[2];
            X[c + 2*Height + offset]=   C2 * SP3 + C6 * SP4;
            X[c + 6*Height + offset]= - C2 * SP4 + C6 * SP3;
            X[c + 1*Height + offset]= C1 * in[4] + C3 * in[5] 
                    + C5 * in[6] + C7 * in[7];
            X[c + 3*Height + offset]= C3 * in[4] - C7 * in[5] 
                    - C1 * in[6] - C5 * in[7];
            X[c + 5*Height + offset]= C5 * in[4] - C1 * in[5] 
                    + C7 * in[6] + C3 * in[7];
            X[c + 7*Height + offset]= C7 * in[4] - C5 * in[5] 
                    + C3 * in[6] - C1 * in[7];
            
        }
    }
    
    return;
}
