/*
   Copyright (C) 2013  PIERRE Fabien.
   All rights reserved.
   Read license.txt.
*/

#include "matrix.h"
#include "mex.h"

#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#define pre_calc (c*Height + offset)

/*************************************************************************
 *
 * AUTEUR : Fabien PIERRE.
 * DATE   : 9 juillet 2013.
 *
 *
 * Compilation :
 
  % linux
  clc
  mex CFLAGS="\$CFLAGS -std=c99" block_IDCT.c -output block_IDCT
 
  % windows
  clc
  mex  block_IDCT.c -output block_IDCT
 
 ************************************************************************/

void mexFunction(int nlhs, mxArray *plhs[]       ,
        int nrhs, const mxArray *prhs[]  )
{
    /* Declarations.*/
    double *x,*X; /* , *WT;*/
    int ndim, i,j;
    const int *dims;
    double C1, C2, C3, C4, C5, C6, C7, SP1, SP2, SP3, SP4; 
    double in[8];
    
    /* Verification du nombre d'arguments.*/
    if(nrhs != 1)
    {
        mexErrMsgIdAndTxt("block_IDCT:nrhs","1 inputs required.");
    }
    if(nlhs != 1)
    {
        mexErrMsgIdAndTxt("block_IDCT:nlhs","1 output required.");
    }
    
    /* Gestion entrees-sorties.*/
    ndim= mxGetNumberOfDimensions(prhs[0]);
    dims= mxGetDimensions(prhs[0]);
    
    C1= 0.490392640201615 ;
    C2= 0.461939766255643 ;
    C3= 0.415734806151273 ;
    C4= 0.353553390593274 ;
    C5= 0.277785116509801 ;
    C6= 0.191341716182545 ;
    C7= 0.097545161008064 ;
    
    X = (const double*)(mxGetPr(prhs[0]));
    
    int height = dims[0];
    int width  = dims[1];
    
    int Height = 8 * (height / 8);
    int Width  = 8 * (width  / 8);
        
    int dims2[2];
    dims2[0] = Height  ;
    dims2[1] = Width  ;
    
    int height_Block = Height / 8;
    int width_Block = Width / 8;
    int length_Block = height_Block *width_Block ;
    
        
    plhs[0]= mxCreateNumericArray(ndim, dims2 ,
            mxDOUBLE_CLASS, mxREAL     ) ;
    x= (double*)mxGetPr(plhs[0]);
    for(int lin=0; lin < Height; lin++){
        for(int col=0; col < Width; col++){
            X[ lin + col * Height]= X[lin + col * height];
        }
    }
    
    int X_Block;
    int Y_Block;
    int offset;
    
    for(int index= 0; index<length_Block; index ++){
        
        X_Block= index % height_Block;
        Y_Block= (index - X_Block) / height_Block;
        offset= X_Block * 8 + 64 * height_Block * Y_Block;
        
        for(int c=0; c< 8; c++){
          //  pre_calc= c*Height + offset;
            SP1= C4 * ( X[0 + pre_calc] +  X[4 + pre_calc] );
            SP2= C4 * ( X[0 + pre_calc] -  X[4 + pre_calc] );
            SP3= C6 * X[6 + pre_calc] + C2 * X[2 + pre_calc];
            SP4= C6 * X[2 + pre_calc] - C2 * X[6 + pre_calc];
            
            in[0] =  SP1 + SP3;
            in[1] =  SP2 + SP4;
            in[2] = -SP4 + SP2;
            in[3] =  SP1 - SP3;
            
            in[4]= C1 * X[1 + pre_calc] + C3 * X[3 + pre_calc]
                    + C5 * X[5 + pre_calc] + C7 * X[7 + pre_calc];
            in[5]= C3 * X[1 + pre_calc] - C7 * X[3 + pre_calc]
                    - C1 * X[5 + pre_calc] - C5 * X[7 + pre_calc];
            in[6]= C5 * X[1 + pre_calc] - C1 * X[3 + pre_calc]
                    + C7 * X[5 + pre_calc] + C3 * X[7 + pre_calc];
            in[7]= C7 * X[1 + pre_calc] - C5 * X[3 + pre_calc]
                    + C3 * X[5 + pre_calc] - C1 * X[7 + pre_calc];
            
            for(int k=0; k < 4; k++){
                x[k     + pre_calc] = in[ k ] + in[ k + 4 ];
                x[7-k  +  pre_calc] = in[ k ] - in[ k + 4 ];
            }
        }
        
        for(int c=0; c < 8; c++){
            
            SP1= C4 * ( x[c + 0*Height + offset] 
                    +  x[c + 4*Height + offset] );
            SP2= C4 * ( x[c + 0*Height + offset] 
                    -  x[c + 4*Height + offset] );
            SP3= C6 * x[c + 6*Height + offset] 
                    + C2 * x[c + 2*Height + offset];
            SP4= C6 * x[c + 2*Height + offset] 
                    - C2 * x[c + 6*Height + offset];
            
            in[0] =  SP1 + SP3;
            in[1] =  SP2 + SP4;
            in[2] = -SP4 + SP2;
            in[3] =  SP1 - SP3;
            
            in[4]= C1 * x[c + 1*Height + offset] 
                    + C3 * x[c + 3*Height + offset]
                    + C5 * x[c + 5*Height + offset] 
                    + C7 * x[c + 7*Height + offset];
            in[5]= C3 * x[c + 1*Height + offset] 
                    - C7 * x[c + 3*Height + offset]
                    - C1 * x[c + 5*Height + offset] 
                    - C5 * x[c + 7*Height + offset];
            in[6]= C5 * x[c + 1*Height + offset] 
                    - C1 * x[c + 3*Height + offset]
                    + C7 * x[c + 5*Height + offset] 
                    + C3 * x[c + 7*Height + offset];
            in[7]= C7 * x[c + 1*Height + offset] 
                    - C5 * x[c + 3*Height + offset]
                    + C3 * x[c + 5*Height + offset] 
                    - C1 * x[c + 7*Height + offset];
            
            for(int k=0; k < 4; k++){
                x[c + k *Height + offset] = in[ k ] + in[ k + 4 ];
                x[c + Height*(7-k) + offset] = in[ k ] - in[ k + 4 ];
            }
        }
    }
    
    return;
}
